import QtQuick 2.9
import QtQuick.Layouts 1.1
import QtQuick.Controls 2.3 as QQC2
import org.kde.kirigami 2.4 as Kirigami
import org.kde.kquickcontrols 2.0 as KQuickControls
import org.kde.plasma.components 3.0 as PlasmaComponents3
import org.kde.kirigamiaddons.labs.mobileform 0.1 as MobileForm
import org.kde.kcm 1.5 as KCM

import DeviceModel 1.0
import IPCClient 1.0
import PolkitIPCClient 1.0
import ServiceHelper 1.0

KCM.SimpleKCM {
    id: root

    property bool isConnected: ( kcm.targetConnect.ipcIsConnected ?? false ) && serviceHelper.isTurnOn

    function checkConnection() {
        root.isConnected = ( kcm.targetConnect.ipcIsConnected ?? false ) && serviceHelper.isTurnOn;
    }

    ServiceHelper {
        id: serviceHelper
        onIsTurnOnChanged: {
            if(isTurned)
            {
                log.append(i18n("USBGuard service started"));
                kcm.targetConnect.ipcConnect();
                kcm.load();
            }
            else
            {
                log.append(i18n("USBGuard service stopped"));
                kcm.deviceModel.clear();
            }
            checkConnection();
        }
    }

    ColumnLayout {
        MobileForm.FormCard {
            Layout.fillWidth: true
            contentItem: ColumnLayout {
                spacing: 0
                Kirigami.InlineMessage {
                    Kirigami.FormData.isSection: true
                    Layout.fillWidth: true
                    type: Kirigami.MessageType.Error
                    text: i18n("No access to the usbguard-dbus.service")
                    visible: !isConnected
                }

                MobileForm.FormCardHeader {
                    title: i18n("Usbguard service")
                }

                MobileForm.FormSwitchDelegate {
                    id: turnOnSwitch
                    text: i18n("Turn on the service right now")
                    checked: serviceHelper.isTurnOn
                    onClicked: serviceHelper.isTurnOn = checked
                    enabled: !serviceHelper.appliesChanges
                }

                MobileForm.FormSwitchDelegate {
                    id: enableSwitch
                    text: i18n("Enable the service at system startup")
                    checked: serviceHelper.isEnable
                    onClicked: serviceHelper.isEnable = checked
                    enabled: !serviceHelper.appliesChanges
                }

                MobileForm.AbstractFormDelegate {
                    Layout.fillWidth: true
                    background: Item {}
                    clip: true
                    leftPadding: 0
                    topPadding:  0
                    bottomPadding: 0
                    rightPadding: 0

                    contentItem: ColumnLayout {
                        spacing: 0

                        MobileForm.FormButtonDelegate {
                            id: showLogButton
                            text: i18n("Show log")
                            visible: true
                            onClicked: {
                                logWrapper.visible = !logWrapper.visible;
                                text = logWrapper.visible ? i18n("Hide log") : i18n("Show log");
                            }

                            background: Rectangle {
                                id: showLogButtonBackground
                                color: "transparent"
                                SequentialAnimation {
                                    id: focusLogAnim
                                    loops: 3
                                    PropertyAnimation {
                                        target: showLogButtonBackground
                                        property: "color"
                                        to: Qt.rgba(Kirigami.Theme.textColor.r, Kirigami.Theme.textColor.g, Kirigami.Theme.textColor.b, 0.2)
                                        duration: Kirigami.Units.veryLongDuration

                                    }
                                    PropertyAnimation {
                                        target: showLogButtonBackground
                                        property: "color"
                                        to: "transparent"
                                        duration: Kirigami.Units.veryLongDuration
                                    }
                                }
                            }
                        }
                        MobileForm.FormDelegateSeparator {
                            visible: logWrapper.visible
                        }
                        Flickable {
                            id: logWrapper
                            flickableDirection: Flickable.VerticalFlick
                            implicitHeight: 200
                            implicitWidth: parent.width
                            visible: false

                            QQC2.TextArea.flickable: QQC2.TextArea {
                                id: log
                                enabled: false
                                background: Item {}

                                implicitWidth: parent.width
                                wrapMode: QQC2.TextArea.WordWrap


                                // scroll log to bottom
                                onTextChanged: {
                                    cursorPosition = length-1;
                                    if (!logWrapper.visible) // notify that we have new notifications
                                        focusLogAnim.restart();
                                }
                            }

                            QQC2.ScrollBar.vertical: QQC2.ScrollBar {}
                        }
                    }
                }
            }
        }

        MobileForm.FormCard {
            Layout.fillWidth: true
            visible: kcm.deviceModel && deviceRepeater.count > 0
            contentItem: ColumnLayout {
                spacing: 0

                MobileForm.FormCardHeader {
                    title: i18n("Units")
                }

                Repeater {
                    id: deviceRepeater
                    Layout.fillWidth: true
                    enabled: isConnected
                    model: kcm.deviceModel

                    delegate: MobileForm.AbstractFormDelegate {
                        id: unitDelegate
                        property var dataModel: model

                        Layout.fillWidth: true
                        clip: true
                        leftPadding: 0
                        topPadding:  0
                        bottomPadding: 0
                        rightPadding: 0

                        contentItem: ColumnLayout {
                            id: contentUnit
                            property var dataModel: model

                            spacing: 0

                            RowLayout {
                                spacing: Kirigami.Units.smallSpacing
                                MobileForm.FormTextDelegate {
                                    Layout.fillWidth: true
                                    text: model.Name
                                    description: extendedInfo.visible ? "" : i18n("USB id: %1, Serial: %2", model.DeviceId || i18n("< Unknown >"), model.Serial || i18n("< Unknown >"))
                                    onClicked: extendedInfo.visible = !extendedInfo.visible;
                                    background: Item {}
                                }

                                QQC2.ComboBox {
                                    Layout.rightMargin: Kirigami.Units.gridUnit
                                    model: [i18n("Allow"), i18n("Block"), i18n("Rejected")]

                                    currentIndex: {
                                        switch(dataModel.Target)
                                        {
                                        case "allow":
                                            return 0;
                                        case "block":
                                            return 1;
                                        case "reject":
                                            return 2;
                                        default:
                                            console.error(i18nc("Unknown target type: \"%1\"", dataModel.Target));
                                            return -1;
                                        }
                                    }

                                    onCurrentIndexChanged: {
                                        switch(currentIndex)
                                        {
                                        case 0:
                                            kcm.deviceModel.updateRequestedTarget(dataModel.RuleId, "allow");
                                            break;
                                        case 1:
                                            kcm.deviceModel.updateRequestedTarget(dataModel.RuleId, "block");
                                            break;
                                        case 2:
                                            kcm.deviceModel.updateRequestedTarget(dataModel.RuleId, "reject");
                                            break;
                                        default:
                                            console.error(i18nc("Unknown target index: \"%1\"", currentIndex));
                                            break;
                                        }
                                    }
                                }
                            }

                            ColumnLayout {
                                id: extendedInfo
                                visible: false
                                spacing: 0

                                MobileForm.FormTextDelegate {
                                    text: i18n("Id")
                                    description: model.DeviceId || i18n("< Unknown >")
                                }
                                MobileForm.FormTextDelegate {
                                    text: i18n("Serial")
                                    description: model.Serial || i18n("< Unknown >")
                                }
                                MobileForm.FormTextDelegate {
                                    text: i18n("Port")
                                    description: model.DevicePort || i18n("< Unknown >")
                                }
                                MobileForm.FormTextDelegate {
                                    text: i18n("Interfaces")
                                    description: model.Interfaces || i18n("< Hidden >")
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    Connections {
        target: IPCClient

        onConnectingStart: { log.append(i18n("Connecting...")) }

        onConnectingFinish: {
            if(connectingSuccess) {
                log.append(i18n("Connected"));
            } else {
                log.append(i18n("Not connected. Trying with polkit"));
                kcm.usePolkit(true);
                kcm.targetConnect.ipcConnect();
            }
        }

        onErrorMessage: {
            log.append(error);
        }

        onNotifyMessage: {
            log.append(message);
        }
    }

    actions.main: Kirigami.Action {
        icon.name: "dialog-ok"
        text: i18n("Save")
        enabled: true
        onTriggered: kcm.save()
    }

    Connections {
        target: PolkitIPCClient

        onConnectingStart: log.append(i18n("Connecting polkit..."))
        onErrorMessage: log.append(error)
        onNotifyMessage: log.append(message)

        onConnectingFinish: {
            if(connectingSuccess) {
                checkConnection();
                kcm.load();
                log.append(i18n("Polkit connected"))
            } else {
                log.append(i18n("Polkit couldn't connect."));
            }
        }

    }

    Connections {
        target: serviceHelper

        function onTurnOnChanged(val) {
            if (turnOnSwitch.checked !== val)
                turnOnSwitch.checked = val;
        }

        function onEnableChanged(val) {
            if (enableSwitch.checked !== val)
                enableSwitch.checked = val;
        }
    }

    Component.onCompleted: {
        kcm.usePolkit(false);
        kcm.targetConnect.ipcConnect();
    }
}
